import torch
from hubconf import silero_vad
import numpy as np
import pyaudio
import wave
import time
import whisper
import os

# 配置参数
SAMPLING_RATE = 16000
CHUNK = 512
MAX_RECORD_TIME = 10
USE_ONNX = True
FORCE_ONNX_CPU = False
THRESHOLD = 0.8
output_dir = os.path.join(os.path.dirname(__file__), 'output')

# 加载语音活动检测模型
print("加载模型...")
model, utils = silero_vad(USE_ONNX, FORCE_ONNX_CPU)
(get_speech_timestamps, save_audio, read_audio, VADIterator, collect_chunks) = utils
vad_iterator = VADIterator(model)
print("模型加载完成")

# 初始化音频录制
audio = pyaudio.PyAudio()
stream = audio.open(
    format=pyaudio.paInt16,
    channels=1,
    rate=SAMPLING_RATE,
    input=True,
    frames_per_buffer=CHUNK
)

print("开始录音，按Ctrl+C取消...")

# 初始化录音变量
audio_chunks = []
speech_start_time = None
speech_end_time = None
is_recording = False
max_record_time_reached = False
human_id = 1
first_start_time = 0

def transcribe_wav_to_text(wav_filename):
    """使用 Whisper 模型将 WAV 文件转录为文本"""
    print("使用 Whisper 模型转录音频...")
    model = whisper.load_model("tiny")
    result = model.transcribe(wav_filename)
    text = result["text"]
    print(f"转录文本：{text}")

def save_wav(audio_chunks, start_time, end_time, status):
    """保存音频片段并进行语音转文本"""
    print("保存音频文件...")
    
    # 计算音频数据的总长度并保存为 WAV 文件
    num_frames = sum(len(chunk) for chunk in audio_chunks)
    audio_data = np.concatenate([np.frombuffer(chunk, dtype=np.int16) for chunk in audio_chunks])
    wav_filename = f"{output_dir}/output_{int(start_time)}_{status}.wav"
    
    with wave.open(wav_filename, 'wb') as wf:
        wf.setnchannels(1)
        wf.setsampwidth(2)
        wf.setframerate(SAMPLING_RATE)
        wf.writeframes(audio_data)
    
    print(f"音频保存为 {wav_filename}，时长从 {start_time:.2f} 秒到 {end_time:.2f} 秒，状态：{status}")

    # 调用语音转文字函数
    transcribe_wav_to_text(wav_filename)

try:
    while True:
        # 读取音频数据
        data = stream.read(CHUNK, exception_on_overflow=False)
        np_array = np.frombuffer(data, dtype=np.int16).copy()
        audio_chunk = torch.from_numpy(np_array).float()

        # 检测语音活动
        with torch.no_grad():
            speech_prob = model(audio_chunk.view(1, -1), SAMPLING_RATE).item()
            
            if speech_prob > THRESHOLD:
                if speech_start_time is None:
                    speech_start_time = time.time()
                    if first_start_time == 0:
                        first_start_time = speech_start_time
                        print("first_start_time:", first_start_time)
                    print("开始检测到语音")
                last_speech_end_time = None
            else:
                if speech_start_time is not None:
                    if last_speech_end_time is None:
                        last_speech_end_time = time.time()
                    
                    if time.time() - last_speech_end_time >= 1.0:
                        current_time = time.time()
                        print("停止检测到语音，保存录音")
                        save_wav(audio_chunks, speech_start_time, current_time, 0)
                        audio_chunks = []
                        speech_start_time = None
                        last_speech_end_time = None

            if speech_start_time is not None:
                audio_chunks.append(data)

except KeyboardInterrupt:
    print("\n录音结束")

finally:
    if speech_start_time is not None:
        current_time = time.time()
        print("异常结束，保存当前录音")
        save_wav(audio_chunks, speech_start_time, current_time, 0)

    stream.stop_stream()
    stream.close()
    audio.terminate()
    vad_iterator.reset_states()
    print("资源释放完成")
